@@ -19,7 +19,7 @@ use ide_db::{
 use syntax::{SmolStr, SyntaxKind, TextRange};
 
 use crate::{
-    context::{PathCompletionContext, PathKind},
+    context::{PathCompletionCtx, PathKind},
     item::{CompletionRelevanceTypeMatch, ImportEdit},
     render::{enum_variant::render_variant, function::render_fn, macro_::render_macro},
     CompletionContext, CompletionItem, CompletionItemKind, CompletionRelevance,
@@ -28,11 +28,15 @@ use crate::{
 #[derive(Debug)]
 pub(crate) struct RenderContext<'a> {
     completion: &'a CompletionContext<'a>,
+    is_private_editable: bool,
 }
 
 impl<'a> RenderContext<'a> {
-    pub(crate) fn new(completion: &'a CompletionContext<'a>) -> RenderContext<'a> {
-        RenderContext { completion }
+    pub(crate) fn new(
+        completion: &'a CompletionContext<'a>,
+        is_private_editable: bool,
+    ) -> RenderContext<'a> {
+        RenderContext { completion, is_private_editable }
     }
 
     fn snippet_cap(&self) -> Option<SnippetCap> {
@@ -47,6 +51,10 @@ impl<'a> RenderContext<'a> {
         self.completion.source_range()
     }
 
+    fn completion_relevance(&self) -> CompletionRelevance {
+        CompletionRelevance { is_private_editable: self.is_private_editable, ..Default::default() }
+    }
+
     fn is_deprecated(&self, def: impl HasAttrs) -> bool {
         let attrs = def.attrs(self.db());
         attrs.by_key("deprecated").exists() || attrs.by_key("rustc_deprecated").exists()
@@ -226,7 +234,7 @@ fn render_resolution_(
     // Add `<>` for generic types
     let type_path_no_ty_args = matches!(
         ctx.completion.path_context,
-        Some(PathCompletionContext { kind: Some(PathKind::Type), has_type_args: false, .. })
+        Some(PathCompletionCtx { kind: Some(PathKind::Type), has_type_args: false, .. })
     ) && ctx.completion.config.add_call_parenthesis;
     if type_path_no_ty_args {
         if let Some(cap) = ctx.snippet_cap() {
@@ -582,6 +590,7 @@ fn main() { let _: m::Spam = S$0 }
                             ),
                             is_local: false,
                             is_op_method: false,
+                            is_private_editable: false,
                             exact_postfix_snippet_match: false,
                         },
                         trigger_call_info: true,
@@ -603,6 +612,7 @@ fn main() { let _: m::Spam = S$0 }
                             ),
                             is_local: false,
                             is_op_method: false,
+                            is_private_editable: false,
                             exact_postfix_snippet_match: false,
                         },
                     },
@@ -689,6 +699,7 @@ fn foo() { A { the$0 } }
                             ),
                             is_local: false,
                             is_op_method: false,
+                            is_private_editable: false,
                             exact_postfix_snippet_match: false,
                         },
                     },
@@ -1354,7 +1365,7 @@ fn main() {
     }
 
     #[test]
-    fn op_method_relevances() {
+    fn op_function_relevances() {
         check_relevance(
             r#"
 #[lang = "sub"]
@@ -1367,7 +1378,30 @@ fn foo(a: u32) { a.$0 }
             expect![[r#"
                 me sub(…) (as Sub) [op_method]
             "#]],
-        )
+        );
+        check_relevance(
+            r#"
+struct Foo;
+impl Foo {
+    fn new() -> Self {}
+}
+#[lang = "eq"]
+pub trait PartialEq<Rhs: ?Sized = Self> {
+    fn eq(&self, other: &Rhs) -> bool;
+    fn ne(&self, other: &Rhs) -> bool;
+}
+
+impl PartialEq for Foo {}
+fn main() {
+    Foo::$0
+}
+"#,
+            expect![[r#"
+                fn new() []
+                me eq(…) (as PartialEq) [op_method]
+                me ne(…) (as PartialEq) [op_method]
+            "#]],
+        );
     }
 
     #[test]
